'use strict';

exports.__esModule = true;
exports.tokenRegex = tokenRegex;
exports.parseToken = parseToken;
exports.serializeToken = serializeToken;
exports.extractTokens = extractTokens;

var _compare = require('./compare');

function tokenRegex() {
  var opts = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

  var qtfr = opts.partial ? '*' : '+';

  return new RegExp('(?!^|\\(|\\s)*' + // find beginning of token
  '([-]+)?' + // capture prepended negation character
  '([\\w.$]+)' + ( // the attribute name
  '' + (opts.partial ? '?' : '')) + ( // assume it's a token, even with no attribute
  ':' + (opts.partial ? '?' : '')) + // assume it's a token, even with no colon
  '(?!:)' + // make sure colon isn't repeated
  '([><=]*)' + ( // the operators
  '(?:(")(.' + qtfr + '?)(\\*)?"|([^\\s()*"]' + qtfr + '))') + ( // the attribute value, checking for quotes
  '' + (opts.partial ? '?' : '')) + // whether attribute value can be empty
  '(\\*)?' + // capture appended wildcard
  '(?!\\s|\\)|$)*', // find the end of the token
  'g');
}

function parseToken(value) {
  var attributes = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : [];
  var nameKey = arguments[2];

  var results = Array.isArray(value) ? value : tokenRegex({ partial: true }).exec(value);

  if (!results || !results.length) {
    return {};
  }

  var tokenData = {
    fullToken: results[0],
    attributeName: results[2],
    attributeValue: results[5] || results[7],
    prepended: results[1] || '',
    operator: results[3],
    negated: results[0].indexOf('-') > -1,
    quoted: Boolean(results[4]),
    wildcard: Boolean(results[6] || results[8])
  };

  if (attributes) {
    var attribute = attributes.find(function (attr) {
      return (0, _compare.compare)(attr[nameKey], tokenData.attributeName);
    });

    if (attribute) {
      tokenData.attributeNameValid = true;
      tokenData.attributeValueValid = true;

      if (Array.isArray(attribute.enumerations)) {
        tokenData.attributeValueValid = attribute.enumerations.findIndex(function (v) {
          return (0, _compare.compareFuzzy)(tokenData.attributeValue, v);
        }) > -1;
      }
    }
  }

  return tokenData;
}

function serializeToken(token) {
  var _ref = token || {},
      _ref$prepended = _ref.prepended,
      prepended = _ref$prepended === undefined ? '' : _ref$prepended,
      _ref$attributeName = _ref.attributeName,
      attributeName = _ref$attributeName === undefined ? '' : _ref$attributeName,
      _ref$attributeValue = _ref.attributeValue,
      attributeValue = _ref$attributeValue === undefined ? '' : _ref$attributeValue,
      _ref$operator = _ref.operator,
      operator = _ref$operator === undefined ? '' : _ref$operator;

  return '' + prepended + attributeName + ':' + operator + attributeValue;
}

function extractTokens(value, attributes, nameKey) {
  var positions = [];
  var regex = tokenRegex();

  var result = void 0;
  while ((result = regex.exec(value)) !== null) {
    if (attributes) {
      var parsed = parseToken(result, attributes, nameKey);

      if (!parsed.attributeNameValid || !parsed.attributeValueValid) {
        continue;
      }
    }

    positions.push([result.index, // start position
    regex.lastIndex // end position
    ]);
  }

  return positions;
}